import augmentation as am

class Flip(am.Augmentatation):
    def __init__(self, augment):
        self.augment = augment
        
    def execute():
        self.augment.execute()
        
        
        
    def _augmentation(self, image, segmentation_mask, augmente_type):
        image_aug = tf.image.flip_left_right(image)
        segmentation_mask_aug = tf.image.flip_left_right(segmentation_mask)
        return image_aug, segmentation_mask_aug


from abc import *


class Augmentation(metaclass=ABCMeta):
    def __init__(self, global_variable, log):
        self.gv = global_variable
        self.log = log
    
    @abstractmethod
    def execute():
        pass


import os
import sys
import json
import cv2 as cv
import numpy as np
from io import BytesIO
import tensorflow as tf
import tensorflow_datasets as tfds
from google.cloud import storage
from google.cloud.storage import Blob
from sklearn.model_selection import train_test_split
from .. import global_variable as gv
from . import dataset_mgmt as dsmgmt

import pandas as pd


class MiddleDatasetMgmt(dsmgmt.DatasetMgmt):
    
    def __init__(self, global_variable, log):
        super(MiddleDatasetMgmt, self).__init__(global_variable, log)
        self.client = storage.Client(self.gv.PROJECT_ID)
        
        
    def _bytes_feature(self, value):
        """Returns a bytes_list from a string / byte."""
        if isinstance(value, type(tf.constant(0))):
            value = value.numpy() # BytesList won't unpack a string from an EagerTensor.
        return tf.train.Feature(bytes_list=tf.train.BytesList(value=[value]))

    
    def _float_feature(self, value):
        """Returns a float_list from a float / double."""
        return tf.train.Feature(float_list=tf.train.FloatList(value=[value]))

    
    def _int64_feature(self, value):
        """Returns an int64_list from a bool / enum / int / uint."""
        return tf.train.Feature(int64_list=tf.train.Int64List(value=[value]))
    
    
    def _make_example_image(self, image_bytes, label_bytes): #write
        feature = {'image': self._bytes_feature(image_bytes),
                   'segmentation_mask': self._bytes_feature(label_bytes)}
        return tf.train.Example(features=tf.train.Features(feature=feature))
    
    
    def _make_example_metadata(self, label, value): #write
        feature = {label: self._int64_feature(value)}
        return tf.train.Example(features=tf.train.Features(feature=feature))
    
    
    def _get_dataset_metadata(self, bucket):
        blob = bucket.blob(self.gv.DEV_DATASET_METADATA_NAME) if self.gv.DEV_MODE else bucket.blob(self.gv.DATASET_METADATA_NAME)
        
        if self.gv.DEV_MODE and None is blob:
            print(f"{self.gv.DEV_DATASET_METADATA_NAME} is not exist")
            blob = bucket.blob(self.gv.DATASET_METADATA_NAME)
        
        with blob.open("r") as f:
            metadata_list = json.loads(f.read())
            
        return metadata_list
    
    
    def _parse_image_function(self, example_proto): # read
        image_feature_description = {
            'image': tf.io.FixedLenFeature([], tf.string),
            'segmentation_mask':tf.io.FixedLenFeature([], tf.string),
        }
        return tf.io.parse_single_example(example_proto, image_feature_description)
                           
                           
    def _parse_image_count_function(self, example_proto): # read
        image_count_feature_description = {
            'count': tf.io.FixedLenFeature([], tf.int64),
        }
        return tf.io.parse_single_example(example_proto, image_count_feature_description)
    
    
    def _augmentation(self, image, segmentation_mask, augmente_type):
        image_aug = tf.image.flip_left_right(image)
        segmentation_mask_aug = tf.image.flip_left_right(segmentation_mask)
        return image_aug, segmentation_mask_aug
    
    ###
    
    def make_bucket(self, bucket_name):
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        
        try:
            self.client.get_bucket(bucket_name) #lockup_bucket을 대신 사용할 수도 있음
            self.log.info(f"{bucket_name} bucket already exists")
        except:
            self.log.info(f"{bucket_name} bucket doesn't exists")
            bucket = self.client.bucket(bucket_name)
            bucket.storage_class = "STANDARD"
            
            #mb : make bucket (PROJECT_ID와 REGION에 종속된 DATA_BUCKET 생성)
            #!gsutil mb -p $self.gv.PROJECT_ID -c standard -l $self.gv.REGION gs://$gv.DATA_BUCKET 
            new = self.client.create_bucket(bucket, project=self.gv.PROJECT_ID, location=self.gv.REGION)
            self.log.info(f"Created bucket {new.name} in {new.location} with storage class {new.storage_class}")  
        
    
    def download(self, dataset_name):
        self.log.info("under contruction")
    
    
    def upload(self, bucket_name):
        self.log.info("under contruction")
    
    
    ###
    
    def read(self, bucket_name): #TFRecord format
        self.log.info("under contruction")
    
    
    def create_dev_dataset_metadata(self, bucket_name, start_idx, end_idx):
        bucket = self.client.get_bucket(bucket_name)
        metadata_list = self._get_dataset_metadata(bucket)
        
        blob = Blob(self.gv.DEV_DATASET_METADATA_NAME, self.client.get_bucket(bucket_name))
        blob.upload_from_string(json.dumps(metadata_list[start_idx:end_idx]))
        
        
    #dataset_begin_idx는 raw dataset source가 다른 것을 적용하는 시점에는 반듯이 삭제해야 함 
    def read_and_split(self, bucket_name, dataset_bucket_idx, data_split): #TFRecord format : storing a sequence of binary records
        test_set = []
        train_val_set = []
        
        bucket = self.client.get_bucket(bucket_name)
        metadata_list = self._get_dataset_metadata(bucket)
        
        print(f"bucket_name={bucket_name}")
        
        for metadata in metadata_list:
            blob = bucket.blob(metadata[self.gv.DATASET_METADATA_KEY_SOURCE_REF]) 
            with blob.open("rb") as f:
                image_bytes = f.read()

            blob = bucket.blob(metadata[self.gv.DATASET_METADATA_KEY_LABEL_REF])
            with blob.open("rb") as f:
                label_bytes = f.read()

            if metadata[self.gv.DATASET_METADATA_KEY_DATA_TYPE] == 'train':
                train_val_set.append(self._make_example_image(image_bytes, label_bytes))
            else:
                test_set.append(self._make_example_image(image_bytes, label_bytes))

        train_set, val_set = train_test_split(train_val_set,
                                              test_size=data_split[1], 
                                              train_size=data_split[0], 
                                              shuffle=False)
        
        return train_set, val_set, test_set
    
    
    def write_dataset_count(self, path, count): ############# 수정필요
        print(f"dataset_len = {count}")
        data = []
        data.append(self._make_example_metadata('count', count))
        self.write(path+"_"+self.gv.DATASET_METADATA_NAME, data)
            
    
    def write(self, path, dataset):  #TFRecord format
        print(f"write_path={path}")
        with tf.io.TFRecordWriter(path, self.gv.DATASET_COMPRESS_TYPE) as writer:
            for tf_example in dataset:
                writer.write(tf_example.SerializeToString())
                
    
    def read_images(self, path):# -> images, count:  #TFRecord format
        image_dataset = tf.data.TFRecordDataset(path, self.gv.DATASET_COMPRESS_TYPE)
        assert isinstance(image_dataset, tf.data.Dataset)
        parsed_image_dataset = image_dataset.map(self._parse_image_function, num_parallel_calls=tf.data.experimental.AUTOTUNE)
                                      
        image_count = tf.data.TFRecordDataset(path+"_"+self.gv.DATASET_METADATA_NAME, self.gv.DATASET_COMPRESS_TYPE)
        assert isinstance(image_count, tf.data.Dataset)
        parsed_image_count = image_count.map(self._parse_image_count_function, num_parallel_calls=tf.data.experimental.AUTOTUNE)
        
        count = [image_count['count'].numpy() for image_count in parsed_image_count]
        return parsed_image_dataset, count[0]
                
        
    def augmente_image(self, parsed_image_dataset, augmente_type):
        train_aug_set = []
        for image_features in parsed_image_dataset:
            img = image_features['image']
            segment_mask = image_features['segmentation_mask']

            train_aug_set.append(self._make_example_image(img, segment_mask))

            image_aug, segment_mask_img_aug = self._augmentation(tf.io.decode_image(img, channels=3), tf.io.decode_png(segment_mask, channels=1), augmente_type)
            train_aug_set.append(self._make_example_image(tf.io.encode_jpeg(image_aug), tf.io.encode_png(segment_mask_img_aug)))
            
        return train_aug_set


    def merge_image_dataset(self, dataset_paths):
        self.log.info(f"merge dataset {dataset_paths[0]} and {dataset_paths[1]}")
        
        dataset1, count = self.read_images(dataset_paths[0])
        print(f"dataset1={dataset1}, {count}\n\n")
        dataset2, count = self.read_images(dataset_paths[1])
        print(f"dataset2={dataset2}, {count}\n\n")
        dataset = dataset1.concatenate(dataset2)
        print(f"dataset_concatenate={dataset}\n\n")
        # dataset = pd.merge(dataset, dataset_append)
        # print(f"dataset_concatenate={dataset}\n\n")
        
       
        data_set = [] 
        for image in dataset:
            data_set.append(self._make_example_image(image['image'], image['segmentation_mask']))
            
        return data_set
       
        

# Cloud와 독립되도록 보완하자 => Module 분리
# Download, Upload to GCS
class TensorflowDatasetMgmt(MiddleDatasetMgmt):
    
    def download(self, dataset_name):
        self.dataset = tfds.load(dataset_name, with_info=False)
    
    
    def upload(self, bucket_name):
        self.bucket_name = bucket_name
        
        dataset_label = self.__upload_dataset_to_bucket(self.dataset['train'], 'train')
        dataset_label.extend(self.__upload_dataset_to_bucket(self.dataset['test'], 'test'))
        self.__upload_dataset_label_to_bucket(dataset_label)
    
    
    # {
    #   "source-ref": "original/train/image/Sphynx_158.jpg",
    #   "label-ref": "original/train/label/Sphynx_158.png",
    #   "data-type": "train"
    # },
    def __upload_dataset_to_bucket(self, dataset, data_type_name):
        dataset_label = []
        bucket = self.client.get_bucket(self.bucket_name)
      
        data_iterator = dataset.as_numpy_iterator()
        for idx, data in enumerate(data_iterator):
            # if self.gv.DEV_MODE and self.gv.DEV_MODE_DATASET_SIZE == idx:
            #     break
            
            f_name = data['file_name'].decode('utf-8')
            
            image = data['image']
            label = data['segmentation_mask']

            blob = Blob(f"original/{data_type_name}/image/{f_name}", bucket)
            blob.upload_from_file(BytesIO(cv.imencode('.jpg', image)[1]))

            blob = Blob(f"original/{data_type_name}/label/{os.path.splitext(f_name)[0]}.png", bucket)
            blob.upload_from_file(BytesIO(cv.imencode('.png', label)[1]))

            dataset_label.append({'source-ref': f"original/{data_type_name}/image/{f_name}", 
                                 'label-ref': f"original/{data_type_name}/label/{os.path.splitext(f_name)[0]}.png", 
                                 'data-type': data_type_name})

        return dataset_label
    
    
    def __upload_dataset_label_to_bucket(self, dataset_label): 
        blob = Blob(self.gv.DATASET_METADATA_NAME, self.client.get_bucket(self.bucket_name))
        blob.upload_from_string(json.dumps(dataset_label))
        
        

# Download, Upload to GCS   
class COCODatasetMgmt(MiddleDatasetMgmt):
    
    def download(self, dataset_name):
        self.log.info("under contruction")
    
    
    def upload(self, bucket_name):
        self.bucket_name = bucket_name
        self.log.info("under contruction")
    
    
    def __upload_dataset_to_bucket(self, dataset, data_type_name):
        self.log.info("under contruction")
        
        
    def __upload_dataset_label_to_bucket(self, dataset_list): 
        self.log.info("under contruction")
    


import augmentation as am
import flip as

# image augmentation 은 type별로 다양한 설정이 필요함
# 다양한 설정들과 사용할 augment type을 하나의 구조체로 입력을 받으면
class AugmentMgmt():
    def get_augment(self, augment_config):
        
        for augconf in augment_config:
            if flip = augconf.type:
                self.augment = flipAugmentation()
            if augconf.type = flip:
            
        if true == flipAugmentation:
            
  
        if () self.augment = ATypeAugmentation(self.augment)
        if () self.augment = BTypeAugmentation(self.augment)
        if () self.augment = CTypeAugmentation(self.augment)

        return self.augment
    
    
    def visualize(original, augmented):
        fig = plt.figure()
        plt.subplot(1,2,1)
        plt.title('Original image')
        plt.imshow(original)

        plt.subplot(1,2,2)
        plt.title('Augmented image')
        plt.imshow(augmented)

        
        
        
        
        
        
        
        
        
        
# tf.image.adjust_brightness	
# tf.image.adjust_contrast	
# tf.image.adjust_gamma	
# tf.image.adjust_hue	
# tf.image.adjust_saturation	
# tf.image.central_crop	Crop factor must be a compile-time constant.
# tf.image.convert_image_dtype	
# tf.image.flip_left_right	
# tf.image.flip_up_down	
# tf.image.grayscale_to_rgb	
# tf.image.hsv_to_rgb	
# tf.image.resize_bilinear	Only align_corners=True is available. size must be a compile-time constant.
# tf.image.random_brightness	
# tf.image.random_contrast	
# tf.image.random_flip_left_right	
# tf.image.random_flip_up_down	
# tf.image.random_hue	
# tf.image.random_saturation	
# tf.image.rgb_to_hsv	
# tf.image.rgb_to_grayscale	
# tf.image.rot90	
# tf.image.total_variation	
# tf.image.transpose_image


import numpy as np
import tensorflow as tf
from matplotlib import pyplot as plt
from .. import global_variable as gv

def display_image_n_label(display_list):
    plt.figure(figsize=(15, 15))
    title = ['Input Image', 'True Mask', 'Predicted Mask']
    
    for i in range(len(display_list)):
        plt.subplot(1, len(display_list), i+1)
        plt.title(title[i])
        plt.imshow(tf.keras.preprocessing.image.array_to_img(display_list[i]))
        plt.axis('off')
    plt.show()

def _parse_protobuf_fn(record_bytes):        
    return tf.io.parse_single_example(            
        record_bytes,            
        {"image": tf.io.FixedLenFeature([], tf.string, default_value=''),
         "segmentation_mask": tf.io.FixedLenFeature([], tf.string, default_value='')}  
    )    

def normalize(input_image, input_mask):
    input_image = tf.cast(input_image, tf.float32) / 255.0
    input_mask -= 1
    return input_image, input_mask


def _decode_image_fn(image):  
        input_image = tf.image.resize(tf.io.decode_image(image['image'], expand_animations=False, channels=3), (128, 128))
        input_mask = tf.image.resize(tf.io.decode_png(image['segmentation_mask'], channels=1), (128, 128))     
        
        input_image, input_mask = normalize(input_image, input_mask)
        return input_image, input_mask




import os
import sys
from abc import *

from .. import global_variable as gv

#GCP, AWS 등 Cloud storage가 대상
#bucket 생성
#dataset loading
#save dataset to gcs
#save meta to blob 


# pipeline
# PROJECT_ID -> REGION -> PIPELINE_ROOT (PIPELINE_BUCKET_PATH / PIPELINE_NAME) / ID / PIPELINE_DISPLAY_NAME / Pipeline Output
# ex) gs://vertex_ai_rokit_tutorial_lsm/vertex-ai-tutorial-pipeline-lsm/1064916457437/vertex-ai-tutorial-pipeline-lsm-20221110071809/oxford-iiit-pet-load_4365227738282328064
# 
# PROJECT_ID -> REGION -> PIPELINE_TEMPLATE_PATH (PIPELINE_BUCKET_PATH / PIPELINE_FOLDER / PIPELINE_NAME.json)
# ex) vertex_ai_rokit_tutorial_lsm/pipelines/vertex-ai-tutorial-pipeline-lsm.json


# dataset source
# PROJECT_ID -> REGION -> gs://DATASET_BUCKET / original / train / image / file_name.jpg ...
# PROJECT_ID -> REGION -> gs://DATASET_BUCKET / original / train / label / file_name.png ...
# PROJECT_ID -> REGION -> gs://DATASET_BUCKET / original / test / image / file_name.jpg ...
# PROJECT_ID -> REGION -> gs://DATASET_BUCKET / original / test / label / file_name.png ...
# PROJECT_ID -> REGION -> gs://DATASET_BUCKET / label.json
# 
# ex) gs://oxford_iiit_pet_3_2_0/original/train/image/Abyssinian_1.jpg
# 
# label.json : image uri info
# [
#   {
#     "source-ref": "original/train/image/Sphynx_158.jpg",
#     "label-ref": "original/train/label/Sphynx_158.png",
#     "data-type": "train"
#   },

    
class DatasetMgmt(metaclass=ABCMeta):
    def __init__(self, global_variable, log):
        self.gv = global_variable
        self.log = log
    
    @abstractmethod
    def make_bucket(self, bucket_name):
        pass
    
    @abstractmethod
    def download(self, dataset_name):
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass
    
    @abstractmethod
    def upload(self, bucket_name):
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass

    @abstractmethod
    def read(self, bucket_name): 
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass

    @abstractmethod
    def read_and_split(self, bucket_name, dataset_bucket_idx, data_split):
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass
    
    @abstractmethod
    def write_dataset_count(self, path, count):
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass
        
    @abstractmethod
    def write(self, path, dataset): 
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass
    
    @abstractmethod
    def read_images(self, path):# -> images, count: 
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass
    
    @abstractmethod
    def augmente_image(self, image_dataset, augmente_type):
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass
    
    @abstractmethod
    def merge_image_dataset(self, dataset_paths):
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass
    
    @abstractmethod
    def create_dev_dataset_metadata(self, bucket_name, start_idx, end_idx):
        self.log.info(f"{sys._getframe(0).f_code.co_name} function called")
        pass
        
    
    
#      #for batch prediction
#     def load_image(datapoint):  
#         input_image = tf.io.decode_image(datapoint['image'], expand_animations = False, channels=3)
#         input_mask = tf.io.decode_png(datapoint['segmentation_mask'], channels=1)

#         input_image = tf.image.resize(input_image, (128, 128))
#         input_mask = tf.image.resize(input_mask, (128, 128))     
        
#         return input_image, input_mask    
    

#     def dataset2jsonl(x_images, path):
#         path = os.path.split(path)[0] #PIPELINE_BUCKET_PATH
#         BATCH_PREDICTION_INSTANCES_FILE = "batch_prediction_instances.jsonl"
#         BATCH_PREDICTION_GCS_SOURCE = (
#             path + "/batch_prediction_instances/" + BATCH_PREDICTION_INSTANCES_FILE
#         )

#         log.info(f"BATCH_PREDICTION_INSTANCES_FILE = {BATCH_PREDICTION_INSTANCES_FILE}")
#         log.info(f"BATCH_PREDICTION_GCS_SOURCE = {BATCH_PREDICTION_GCS_SOURCE}")

#         # Write instances at JSONL
#         with open(BATCH_PREDICTION_INSTANCES_FILE, "w") as f:
#             x_test = [np.asarray(image[0]).astype(np.float32).tolist() for image in x_images]
#             for x in x_test:
#                 f.write(json.dumps(x) + "\n")

#         # Upload to Cloud Storage bucket
#         !gsutil cp $BATCH_PREDICTION_INSTANCES_FILE $BATCH_PREDICTION_GCS_SOURCE
#         log.info(f"Uploaded instances to: {BATCH_PREDICTION_GCS_SOURCE}")
        
      
#     def batch_prediction_input_make(dataset_path):
#         dataset = tf.data.TFRecordDataset(dataset_path, dataset_compression_type).map(_parse_image_function, num_parallel_calls=tf.data.experimental.AUTOTUNE)
#         dataset = dataset.map(load_image, num_parallel_calls=tf.data.experimental.AUTOTUNE)         
#         dataset2jsonl(dataset, dataset_path) 
        


import os    
import sys
from .. import global_variable as gv
from . import gcp_dataset_mgmt as gcp_tf_dsmgmt
from . import aws_dataset_mgmt as aws_tf_dsmgmt
#######################################################################
# import new cloud package
# Example:
# from . import {new_cloud_typ}_dataset_mgmt as {new_cloud_typ}_dsmgmt
#######################################################################


class Storage():
    
    gv = None
    log = None
    dsmgmt = None
    dsmgmt_module = None
    cloud_platform_type = None
    
    
    def __init__(self, global_variable, log):
        self.gv = global_variable
        self.log = log
        
        
    def __set_cloud_dataset_mgmt(self, dsmgmt):
        self.dsmgmt_module = dsmgmt
        self.dsmgmt = self.dsmgmt_module.MiddleDatasetMgmt(self.gv, self.log)

        
    def activate_cloud_platform(self, cloud_platform_type):
        if isinstance(self.gv.eCLOUD_TYPE.GCP, type(cloud_platform_type)):
            self.cloud_platform_type = cloud_platform_type
        else: 
            print("cloud_platform_type error")

    # dataset source type과 deeplearning framework type(tensorflow, pythorch)을 분리해야 함 
    def activate_dataset_source(self, dataset_source_type):
        if isinstance(self.gv.eDATASET_SOURCE_TYPE.TENSRORFLOW, type(dataset_source_type)):
            if self.gv.eDATASET_SOURCE_TYPE.TENSRORFLOW == dataset_source_type:
                self.dsmgmt = self.dsmgmt_module.TensorflowDatasetMgmt(self.gv, self.log)
            if self.gv.eDATASET_SOURCE_TYPE.COCO == dataset_source_type:
                self.dsmgmt = self.dsmgmt_module.COCODatasetMgmt(self.gv, self.log)
            #######################################################################   
            # Add {new_dataset_source_type} activation code in here
            # Example:
            # if self.gv.eDATASET_SOURCE_TYPE.{NEW_DATASET_SOURCE_TYPE} == dataset_source_type:
            #    self.dsmgmt = self.dsmgmt_module.{NewDatasetSourceType}Mgmt(self.gv, self.log)
            #######################################################################
        else: 
            print("dataset_source_type error")
            
            
    def activate_deeplearning_framework(self, dl_framework_type):
        if isinstance(self.gv.eDEEPLEARNING_FRAMEWORK_TYPE.TENSRORFLOW, type(dl_framework_type)):
            if isinstance(self.gv.eCLOUD_TYPE.GCP, type(self.cloud_platform_type)):
                if self.gv.eCLOUD_TYPE.GCP == self.cloud_platform_type:
                    if self.gv.eDEEPLEARNING_FRAMEWORK_TYPE.TENSRORFLOW == dl_framework_type:
                        self.__set_cloud_dataset_mgmt(gcp_tf_dsmgmt)
                    elif self.gv.eDEEPLEARNING_FRAMEWORK_TYPE.PYTORCH == dl_framework_type:
                        self.__set_cloud_dataset_mgmt(gcp_tf_dsmgmt)  #pytorch로 수정 필요
                if self.gv.eCLOUD_TYPE.AWS == self.cloud_platform_type:
                    if self.gv.eDEEPLEARNING_FRAMEWORK_TYPE.TENSRORFLOW == dl_framework_type:
                        self.__set_cloud_dataset_mgmt(aws_tf_dsmgmt)
                    elif self.gv.eDEEPLEARNING_FRAMEWORK_TYPE.PYTORCH == dl_framework_type:
                        self.__set_cloud_dataset_mgmt(aws_tf_dsmgmt) #pytorch로 수정 필요
                #######################################################################   
                # Add {NEW_CLOUD_TYPE} or {new NEW_DL_FRAMEWORK_TYPE} activation code in here
                # Example:
                # if self.gv.eCLOUD_TYPE.{NEW_CLOUD_TYPE} == self.cloud_platform_type:
                #    if self.gv.eDEEPLEARNING_FRAMEWORK_TYPE.{NEW_DL_FRAMEWORK_TYPE} == dl_framework_type:
                #         self.__set_cloud_dataset_mgmt({new_cloud_type}_{NEW_DL_FRAMEWORK_TYPE}_dsmgmt)
                #######################################################################
            else:
                print("cloud_platform_type error")
        else: 
            print("deeplearning_framework error")
   
    
    def activate_platform(self, cloud_type, dl_framework_type, dataset_source_type):
        self.activate_cloud_platform(cloud_type)
        self.activate_deeplearning_framework(dl_framework_type)
        self.activate_dataset_source(dataset_source_type)

        
            
    def make_bucket(self, bucket_name):
        self.dsmgmt.make_bucket(bucket_name)
    
    
    def download_upload_dataset(self, dataset_name, bucket_name):
        self.dsmgmt.make_bucket(bucket_name)
        self.dsmgmt.download(dataset_name)
        self.dsmgmt.upload(bucket_name)

        
    def read_and_split_dataset(self, bucket_name, dataset_bucket_idx, data_split):
        return self.dsmgmt.read_and_split(bucket_name, dataset_bucket_idx, data_split)

        
    def write_dataset(self, path, dataset): 
        print("a")
        self.dsmgmt.write(path, dataset)
        print("b")
        self.dsmgmt.write_dataset_count(path, len(dataset))
        
        
    def read_images(self, path):# -> images, count:
        return self.dsmgmt.read_images(path)
    
    
    def augmentation(self, image_dataset, augmente_type):
        return self.dsmgmt.augmente_image(image_dataset, augmente_type)
    
    
    def merge_image_dataset(self, dataset_paths):
        return self.dsmgmt.merge_image_dataset(dataset_paths)
    
    
    def create_dev_dataset_metadata(self, bucket_name, start_idx, end_idx):
        self.dsmgmt.create_dev_dataset_metadata(bucket_name, start_idx, end_idx)